
import { h, resolveComponent, ref, useSlots, computed, provide, inject } from 'vue'
import componentcustom from './customComponent.vue'
import { cloneDeep } from 'lodash-unified'
import './searchForm.less'
const classNames = 'searchForm'
const SearchForm = {
    props: {
        data: {
            type: Array,
            default: () => []
        },
        onChange: {
            type: Function,
            default: () => { }
        },
        bindProps: {
            type: Object,
            default: () => { }
        },
        formRef: {
            type: Object,
            default: () => { }
        },
        ruleForm: {
            type: Object,
            default: () => {
                return {}
            }
        }
    },
    setup(props, { emit }) {
        const formRef = computed({
            get() {
                return props.formRef
            },
            set(news) {
                emit('update:formRef', news)
                return news
            }
        })
        const ruleForm = computed({
            get() {
                return props.ruleForm
            },
            set(news) {
                emit('update:ruleForm', news)
            }
        })
        const id = ref(0)
        const data = computed(() => props.data)
        provide('formRuleForm', {
            ruleForm,
            id
        })
        const slots = useSlots()
        return () => (<div class={classNames}>
            {
                h(resolveComponent('el-form'), {
                    model: ruleForm,
                    ref: (value) => {
                        formRef.value = value
                    },
                    ...props.bindProps,
                    on: {
                        ...props.events
                    }
                }, {
                    default: () => {
                        return slots.default ? slots.default() : data.value.map((item) => {
                            return <SearchFormItem {...item}></SearchFormItem>
                        })
                    }
                })
            }
        </div>)
    },
}
export const SearchFormItem = {
    props: {
        Component: {
            type: Object,
            default: undefined
        },
        label: {
            type: String,
            default: ''
        },
        prop: {
            type: String,
            default: ''
        },
        defaultValue: {
            type: [Array, String, Object, Number],
            default: undefined
        },
        formSlots: {
            type: Function,
            default: () => { }
        },
        renderHTML: {
            type: Function,
            default: () => { }
        },
        beforeSlot: {
            type: Function,
            default: () => { }
        },
        afterSlot: {
            type: Function,
            default: undefined
        },
        childSlot: {
            type: Array,
            default: undefined
        },
        render: {
            type: Function,
            default: undefined
        },
        formProps: {
            type: Object,
            default: undefined
        },
        componentProps: {
            type: Object,
            default: () => { }
        },
        on: {
            type: Object,
            default: () => { }
        },
        ruleForm: {
            type: Object,
            default: () => { }
        },
        componentSlots: {
            type: Object,
            default: () => { }
        },
        id: {
            type: Number,
            default: 0
        },
        hidden: {
            type: Boolean,
            default: false
        }
    },
    components: {
        componentcustom
    },
    setup(props) {
        const prop = props.prop
        const parent = inject('formRuleForm')
        const defaultValue = computed({
            get() {
                if (!prop) {
                    return ''
                }
                if (parent.ruleForm.value[prop]) {
                    return parent.ruleForm.value[prop]
                } else {
                    parent.ruleForm.value[prop] = props.defaultValue
                    return props.defaultValue
                }

            },
            set(news) {
                if (!prop) {
                    return ''
                }
                parent.ruleForm.value[prop] = news
            }
        })
        parent.id.value = parent.id.value + 1
        return () => {
            if (props.hidden) {
                return
            }
            return (h(resolveComponent('el-form-item'), {
                style: {
                    'margin-bottom': '36px',
                },
                ...props.formProps,
                prop: props.prop,
                label: props.label,

            },
                {
                    ...props.formSlots,
                    default: () => {
                        return <div class={classNames + '_search_item'}>
                            {props.beforeSlot && props.beforeSlot(h)}
                            {
                                props.render ? props.render(h, cloneDeep(props)) : props.prop ? <componentcustom items={props} v-slots={{
                                    ...props.componentSlots
                                }} v-model={defaultValue.value}>
                                </componentcustom> : <componentcustom items={props} v-slots={{
                                    ...props.componentSlots
                                }}>
                                </componentcustom>
                            }
                            {props.afterSlot && props.afterSlot(h)}
                        </div>
                    }
                }))
        }
    }
}
export default SearchForm

