
import { reactive, ref, h, resolveComponent, computed, watch, resolveDirective, withDirectives } from 'vue'
import md5 from 'md5'
import './useTable.less'
import Pagination from './pagination.vue'
import { randomString } from '@/utils/index'
const useTable =  {
	props: 	{
		headerList: {
			type: Array,
			default: () => []
		},
		httprequest: {
			type: Function,
			default: () => {}
		},
		propsObj: {
			type: Object,
			default: () =>  {}
		},
		onEvents: {
			type: Object,
			default: () =>  {} 
		},
		isNeedResultFlag: {
			type: Boolean,
			default: true
		},
		data: {
			type: Array,
			default: () => []
		},
		total: {
			type: Number,
			default: 0
		},
		tableRef: {
			type: Function,
			default: () => {}
		}
	},
	setup(props, { expose }) {
		const headerList = computed(()=>props.headerList)
		const tableMove = resolveDirective('tableMove')
		const isNeedResultFlag = props.isNeedResultFlag 
		const searchObj = reactive({
			page_size: 10,
			page: 1
		})
		const tableList = reactive({
			data: [],
			total:0
		})
		const tableRef = ref()
		watch(() => props.data, (news) => {
			if(!isNeedResultFlag) {
				tableList.data = news
			}
		}, {
			deep: true,
			immediate: true
		})
		watch(() => props.total, (news) => {
			if(!isNeedResultFlag) {
				tableList.total = news
			}
		})
		const changeSize = (size) => {
			searchObj.page = size
			getRequestRes(props.httprequest)
		}
		const sizeChange = (pagesize, value) => {
			searchObj['page_size'] = pagesize
			if(value) {
				return
			}
			getRequestRes(props.httprequest)
		}
		const getRequestRes = async (type) =>  {
			if(type === 'firstPage') {
				searchObj.page = 1
			}
			let res = await props.httprequest(searchObj)
			if(isNeedResultFlag) {
				tableList.data = res.data
				tableList.total = res.total
			}
			
		}
		expose({
			getRequestRes
		})
		watch(() => tableRef.value, (news) => {
			props.tableRef(news)
		}, {
			deep: true
		})
		return () => (
			<div class="table_wrap">
				{
					withDirectives(h(resolveComponent('el-table'), 
					{
						ref: tableRef,
						'v-tableMove': tableRef,
						data: tableList.data,
						'header-cell-style': { background: '#F6F8F9',color: '#7D7F95', 'font-weight': 500 },
						stripe: true,
						...props.propsObj,
						...props.onEvents

					},
					{
						empty: () => <div style="padding': 115px 0">
							暂无数据
						</div>,
						default: () => {
							return headerList.value.map((item) => {
								if(item.hidden) {
									return
								}
								item.id = item.id ? item.id : md5(randomString(12))
								return h(resolveComponent('el-table-column'), {
									key: item.id,
									...item.on,
									...item.props,
								}, 
								{
									header: (scope) => {
										if(item.renderHeaderHtml) {
											return item.renderHeaderHtml(h, {...scope, row: tableList.data[scope.$index] })
										} else {
											return item.props.label
										}
									
									},
									default: (scope) => {
										if(item.renderHTML) {
											return Object.values(scope.row).length && item.renderHTML(h, scope)
										} else {
											return scope.row[item.props.prop]
										}
									}
								})
							})
						}
					}
				),[[tableMove, tableRef]])
				}
				{
					tableList.total > 0 && <Pagination onChange={(size) => changeSize(size)} onSizeChange={(pageSize, value) => sizeChange(pageSize, value)} total={tableList.total} ref="pagination" v-model={ searchObj.page } pageSize={ searchObj.page_size }></Pagination>
				}
			</div>)
	}
}

export default useTable
