import { createApp } from 'vue'
import { commonStore } from '@/store/modules/common'
import axios from 'axios'
import { ElMessage } from 'element-plus'
import App from '@/App.vue'
import VueAxios from 'vue-axios'
import { cancelRequestT } from '@/utils/index'
// import { resetData } from '@/utils/resetData.js'
const app = createApp(App)
app.use(VueAxios, axios);
app.axios.defaults.headers.get = {
    'Content-Type': 'application/json'
}
app.axios.defaults.headers.post = {
    'Content-Type': 'application/json'
}
const mergeObject = (target, source) => {
    for (const key in source) {
        target[key] = source[key];
    }
    return target
}
// const requestObj: {[key: string]: string} = {}
const requestCancelObj = {}
export function sendPost(config,data = {}, otherParams = {}, messageObj){
    const { changeLoading } = commonStore()
    const defaultConfig = mergeObject({ method: 'post', url: '' }, config)
    if(otherParams.loading) {
        changeLoading(otherParams.loading)
    }
    if(otherParams.cancelRequest) {
        if(requestCancelObj[defaultConfig.url]) {
            cancelRequestT(requestCancelObj[defaultConfig.url])
        }
    }
    const dataType = config.method == 'get'? 'params': 'data'
    return new Promise((resolve, reject) => {
        axios({
            ...defaultConfig,
            [dataType]:data,
            cancelToken: new axios.CancelToken((c) => {
                if(otherParams.cancelRequest) {
                    requestCancelObj[defaultConfig.url] = c
                }
            })
        }).then((response) => {
                if(otherParams.loading) {
                    changeLoading('')
                }
                if (response.data.code == 200) {
                    resolve(response.data.data)
                    if(messageObj) {
                        const options = {
                            message: 'success',
                            duration: 3000,
                            type: 'success',
                            ...messageObj,
                        }
                        ElMessage(options)
                    }
                } else if(response.data.code == -1303) {
                    // resetData.call(commonStore)
                } else {
                    ElMessage.error(response.data.message)
                    reject(response.data.msg)
                    throw Error(JSON.stringify(response.data))
                }
            })
            .catch((err) => {
                if(otherParams.loading) {
                    changeLoading('')
                }
                reject(err);
            });
    });
}
const handleStatus = (err) => {
    const response= err.response
    const statusObj = {
        400: '请求错误',
        401: '未授权，请登录',
        403: '拒绝访问',
        404: `请求地址出错:${response.config.url}`,
        408: '请求超时',
        500: '服务器开个小差，请稍后再试',
        501: '服务器开个小差，请稍后再试',
        502: '服务器开个小差，请稍后再试',
        503: '服务器开个小差，请稍后再试',
        504: '服务器开个小差，请稍后再试',
        505 : 'HTTP版本不受支持'
    }
    return statusObj[response.status] || '请求失败，请稍后再试'
}
const errorFn = (err) => {
    if (err.toString().indexOf('timeout') !== -1) {
        ElMessage.error('请求超时，请稍后再试')
    }
    if (err && err.response) {
        ElMessage.error(handleStatus(err))
    }
    return Promise.reject(err)
}
/****此处开始封装不同后台下的axios实例****/
app.axios.interceptors.request.use(
    (config) => {
        const token = localStorage.getItem('userInfo') && JSON.parse(localStorage).getItem('userInfo').token
		if(token) {
			config.headers.token = token
		}
        return config
    },
    (error) => {
        return Promise.reject(error)
    }
)
app.axios.interceptors.response.use((response) => {
    return response
}, errorFn)
